@@ -12,6 +12,8 @@ from spyre_util import (get_spyre_backend_list, get_spyre_model_list,
 from vllm.connections import global_http_connection
 from vllm.distributed import cleanup_dist_env_and_memory
 
+from vllm_spyre import envs
+
 # Running with "fork" can lead to hangs/crashes
 # Specifically, our use of transformers to compare results causes an OMP thread
 # pool to be created, which is then lost when the next test launches vLLM and
@@ -20,10 +22,10 @@ os.environ["VLLM_WORKER_MULTIPROC_METHOD"] = "spawn"
 
 
 def pytest_generate_tests(metafunc):
-    """This hook is called during the collection phase, 
-    specifically when Pytest encounters a test function that 
-    needs parametrization. It receives a metafunc object, 
-    which provides information about the test function and 
+    """This hook is called during the collection phase,
+    specifically when Pytest encounters a test function that
+    needs parametrization. It receives a metafunc object,
+    which provides information about the test function and
     allows for dynamic parametrization."""
 
     # default parameterizations
@@ -117,7 +119,7 @@ def pytest_generate_tests(metafunc):
 def _add_param(param_name: str, param_value, metafunc,
                existing_markers) -> None:
     """helper function to parametrize stuff.
-    We make sure to not parametrize something 
+    We make sure to not parametrize something
     if it exists explicitly on the test"""
     if (param_name in metafunc.fixturenames
             and param_name not in existing_markers):
@@ -132,10 +134,6 @@ def pytest_collection_modifyitems(config, items):
     """ Modify tests at collection time """
     _mark_all_e2e(items)
 
-    _skip_quantized_by_default(config, items)
-
-    _xfail_fp8_on_spyre(items)
-
     _skip_unsupported_compiler_tests(config, items)
 
     sort_tests_for_llm_caching(items)
@@ -154,50 +152,18 @@ def _mark_all_e2e(items):
             item.add_marker(pytest.mark.e2e)
 
 
-def _skip_quantized_by_default(config, items):
-    """Skip tests marked with `quantized` unless the `-m` flag includes it
-    Ref: https://stackoverflow.com/questions/56374588/how-can-i-ensure-tests-with-a-marker-are-only-run-if-explicitly-asked-in-pytest
-
-    This will skip the quantized tests at runtime, but they will still show up
-    as collected when running pytest --collect-only.
-    """
-    markexpr = config.option.markexpr
-    if "quantized" in markexpr:
-        return  # let pytest handle the collection logic
-
-    skip_mymarker = pytest.mark.skip(reason='quantized not selected')
-    for item in items:
-        if "quantized" in item.keywords:
-            item.add_marker(skip_mymarker)
-
-
-def _xfail_fp8_on_spyre(items):
-    """Set an xfail marker on all tests that run quantized models on Spyre
-    hardware.
-    
-    TODO: Relax this to only "spyre and cb" once static batching is supported
-    on spyre.
-    """
-
-    xfail_marker = pytest.mark.xfail(
-        reason="fp8 is not yet supported on Spyre")
-    for item in items:
-        if "quantized" in item.keywords and "spyre" in item.keywords:
-            item.add_marker(xfail_marker)
-
-
 def _skip_unsupported_compiler_tests(config, items):
     """Skip all tests that need compiler changes to run.
     This can be relaxed once the compiler changes are in place
     """
 
     markexpr = config.option.markexpr
-    if "compiler_support_16k" in markexpr:
+    if "compiler_support_32k" in markexpr:
         return  # let pytest handle the collection logic
 
     skip_marker = pytest.mark.skip(reason="Needs compiler changes")
     for item in items:
-        if "spyre" in item.keywords and "compiler_support_16k" in item.keywords:
+        if "spyre" in item.keywords and "compiler_support_32k" in item.keywords:
             item.add_marker(skip_marker)
 
 
@@ -329,3 +295,25 @@ def set_random_seed(request):
     seed = int(func_hash.hexdigest(), 16)
     random.seed(seed)
     yield
+
+
+@pytest.fixture()
+def temporary_enable_log_propagate():
+    """Context manager to temporarily enable log propagation."""
+    import logging
+    logger = logging.getLogger("vllm_spyre")
+    logger.propagate = True
+    yield
+    logger.propagate = False
+
+
+@pytest.fixture()
+def caplog_vllm_spyre(temporary_enable_log_propagate, caplog):
+    # To capture vllm-spyre log, we should enable propagate=True temporarily
+    # because caplog depends on logs propagated to the root logger.
+    yield caplog
+
+
+@pytest.fixture(scope="function", autouse=True)
+def clear_env_cache():
+    envs.clear_env_cache()
\ No newline at end of file
