@@ -5,33 +5,38 @@ from abc import ABC, abstractmethod
 from collections import deque
 from collections.abc import Iterable
 from dataclasses import asdict, dataclass, field
-from typing import (TYPE_CHECKING, Generic, Literal, Optional, TypeVar, Union,
-                    cast, get_args)
+from typing import TYPE_CHECKING, Any, Generic, Optional, TypeVar, Union, cast
 
 import torch
 from torch import nn
 from transformers import (AutoModel, AutoModelForSequenceClassification,
                           AutoTokenizer)
-from vllm.config import DeviceConfig, VllmConfig
+from vllm.config import DeviceConfig, VllmConfig, set_current_vllm_config
 from vllm.forward_context import set_forward_context
 from vllm.logger import init_logger
-from vllm.model_executor.layers.pooler import (ClassifierPooler, Pooler,
-                                               PoolingType)
+from vllm.model_executor.layers.pooler import ClassifierPooler, Pooler
 from vllm.sampling_params import SamplingType
 from vllm.utils import is_pin_memory_available
 from vllm.v1.kv_cache_interface import FullAttentionSpec, KVCacheSpec
 from vllm.v1.outputs import LogprobsTensors, SamplerOutput
+from vllm.v1.pool.metadata import PoolingMetadata
+from vllm.v1.sample.logits_processor import build_logitsprocs
 
 import vllm_spyre.envs as envs_spyre
 import vllm_spyre.utils as utils_spyre
 from vllm_spyre.model_executor.model_loader.spyre import (
     BACKEND_LIST, SpyreAttentionMetadata, SpyreCausalLM)
 from vllm_spyre.platform import SpyrePlatform
+from vllm_spyre.v1.sample.spyre_logits_processor import (
+    build_logitsprocs_for_cb)
 # yapf conflicts with ruff for this block
 # yapf: disable
-from vllm_spyre.v1.worker.spyre_input_batch import (
-    BaseInputBatch, BaseRequestState, PoolingInputBatch, PoolingRequestState,
-    SamplingInputBatch, SamplingRequestState, get_builtin_logits_processors)
+from vllm_spyre.v1.worker.spyre_input_batch import (BaseInputBatch,
+                                                    BaseRequestState,
+                                                    PoolingInputBatch,
+                                                    PoolingRequestState,
+                                                    SamplingInputBatch,
+                                                    SamplingRequestState)
 
 # yapf: enable
 if TYPE_CHECKING:
@@ -44,21 +49,9 @@ else:
     NewRequestData = None
     SamplingMetadata = None
 
+from vllm.tasks import SupportedTask
 from vllm.v1.outputs import EMPTY_MODEL_RUNNER_OUTPUT, ModelRunnerOutput
 
-#############################################################
-# from vllm.tasks import GenerationTask, PoolingTask, SupportedTask
-# TODO: remove when we drop support for 0.10.0
-#############################################################
-GenerationTask = Literal["generate", "transcription"]
-GENERATION_TASKS = get_args(GenerationTask)
-
-PoolingTask = Literal["encode", "embed", "classify", "score"]
-POOLING_TASKS = get_args(PoolingTask)
-
-SupportedTask = Literal[GenerationTask]
-#############################################################
-
 logger = init_logger(__name__)
 
 
@@ -191,7 +184,6 @@ class BaseSpyreModelRunner(ABC, Generic[InputBatchT, RequestStateT,
                                   dtype=torch.long,
                                   device=input_ids_i.device)
 
-            pos_ids_pads = pads
             pos_ids_seq = torch.arange(0,
                                        seq_len,
                                        dtype=torch.long,
@@ -202,7 +194,8 @@ class BaseSpyreModelRunner(ABC, Generic[InputBatchT, RequestStateT,
             # workflow works for nested tensor, this can probably be removed
             padded_input_ids_list.append(torch.cat((pads, input_ids_i)))
             mask_list.append(torch.cat((torch.zeros_like(pads), non_pads)))
-            position_ids_list.append(torch.cat((pos_ids_pads, pos_ids_seq)))
+            position_ids_list.append(
+                torch.cat((torch.zeros_like(pads), pos_ids_seq)))
 
         return padded_input_ids_list, mask_list, position_ids_list
 
@@ -295,9 +288,7 @@ class SpyreModelRunner(BaseSpyreModelRunner[SamplingInputBatch,
         max_pad_length = max(prompt_lens)
         max_decode_length = max(num_decode_tokens)
         self.model = SpyreCausalLM(
-            self.model_config,
-            parallel_config=self.parallel_config,
-            scheduler_config=self.scheduler_config,
+            vllm_config=self.vllm_config,
             max_prompt_length=max_pad_length,
             max_decode_length=max_decode_length,
             rank=self.rank,
@@ -305,9 +296,15 @@ class SpyreModelRunner(BaseSpyreModelRunner[SamplingInputBatch,
 
     def build_input_batch(self) -> SamplingInputBatch:
         # Define logits processors.
-        # TODO(Max): logits processor list should be extensible via engine
-        # constructor argument; for now the list is fixed to builtin processors
-        logits_processors = get_builtin_logits_processors(self.vllm_config)
+
+        custom_logitsprocs = self.vllm_config.model_config.logits_processors
+        logits_processors = \
+            build_logitsprocs(vllm_config=self.vllm_config,
+                              device=self.device,
+                              is_pin_memory=self.pin_memory,
+                              is_pooling_model=False,
+                              custom_logitsprocs=custom_logitsprocs)
+
         return SamplingInputBatch(
             max_num_reqs=self.scheduler_config.max_num_seqs,
             max_model_len=self.model_config.max_model_len,
@@ -554,16 +551,23 @@ class SpyreModelRunner(BaseSpyreModelRunner[SamplingInputBatch,
         if not self.is_driver_worker:
             return EMPTY_MODEL_RUNNER_OUTPUT
 
+        # temporary backward compat code for 0.10.1.1
+        annotations = inspect.getfullargspec(ModelRunnerOutput).annotations
+        extra_args = {}  # type: ignore
+        if ('spec_token_ids' in annotations):
+            extra_args.update({
+                'spec_token_ids': None,
+            })
+
         model_output = ModelRunnerOutput(
             req_ids=list(req_id_to_index.keys()),
             req_id_to_index=req_id_to_index,
             sampled_token_ids=output.sampled_token_ids.tolist(),
-            spec_token_ids=None,
             logprobs=(output.logprobs_tensors.tolists()
                       if output.logprobs_tensors else None),
             prompt_logprobs_dict=prompt_logprobs_dicts,
             pooler_output=[],
-        )
+            **extra_args)
 
         return model_output
 
@@ -823,7 +827,7 @@ class ContinuousBatchingSpyreModelRunner(SpyreModelRunner):
         # Note: Until this feature is supported by the compiler we have to set:
         # n_blocks_warmup = n_blocks_avail
 
-        n_blocks_warmup = self.model.model.get_num_blocks_available()
+        n_blocks_warmup = self.get_total_spyre_blocks()
         self._set_blocks(num_blocks=n_blocks_warmup)
         self.model.model.set_past_key_value_states(num_blocks=n_blocks_warmup)
 
@@ -843,23 +847,55 @@ class ContinuousBatchingSpyreModelRunner(SpyreModelRunner):
         super().complete_warmup()
         # get the number or pages from the actual Spyre card after the warmup
         # and set it accordingly in the model runner and for the kv cache size
-        n_blocks_avail = self.model.model.get_num_blocks_available()
+        n_blocks_avail = self.get_total_spyre_blocks()
         self._set_blocks(num_blocks=n_blocks_avail)
         self.model.model.set_past_key_value_states(num_blocks=n_blocks_avail)
 
     def _set_blocks(self, num_blocks: int) -> None:
-        # overwrite num_blocks for testing scheduler constraints
-        num_blocks_override = SpyrePlatform.get_num_spyre_blocks_override()
-        if num_blocks_override > 0:
-            logger.info(
-                "[WARMUP] Overriding number of KV cache blocks on "
-                "Spyre/CPU to %d.", num_blocks_override)
-            num_blocks = num_blocks_override
-
         # set number of available blocks and populate block_pool
         self.n_blocks = num_blocks
         self.block_pool = deque([i for i in range(self.n_blocks)])
 
+    def get_total_spyre_blocks(self) -> int:
+        """Returns the total number of KV cache blocks available for spyre.
+        This currently returns the number of blocks required for a full-sized
+        batch, which may be greater than the available memory.
+
+        Until a correct available memory api is available, the number of blocks
+        must be overridden with a known good value via
+        cache_config.num_gpu_blocks_override
+        """
+        max_batch_size = self.scheduler_config.max_num_seqs
+        max_model_len = self.scheduler_config.max_model_len
+        block_size = SpyrePlatform.get_block_size()
+        min_req_num_blocks = max_model_len // block_size
+
+        blocks_override = self.cache_config.num_gpu_blocks_override
+        if blocks_override is not None and blocks_override > 0:
+            num_blocks = blocks_override
+        else:
+            num_blocks = max_batch_size * min_req_num_blocks
+
+        # Total number of blocks needs to be a multiple of the batch size
+        # (spyre constraint) so round it down
+        num_blocks = max_batch_size * (num_blocks // max_batch_size)
+
+        if num_blocks < min_req_num_blocks:
+            raise ValueError(
+                f"Number of pages available on Spyre {num_blocks} is not "
+                f"enough to serve the current model (need at least "
+                f"{min_req_num_blocks} pages).")
+
+        max_concurrency = num_blocks * block_size / max_model_len
+        backend = "Spyre" if envs_spyre.VLLM_SPYRE_DYNAMO_BACKEND == 'sendnn' \
+            else "CPU"
+        logger.info("%s KV cache size: %s tokens", backend,
+                    num_blocks * block_size)
+        logger.info("Maximum concurrency for %s tokens per request: %.2fx",
+                    str(max_model_len), max_concurrency)
+
+        return num_blocks
+
     def update_states(self, scheduler_output):
 
         super().update_states(scheduler_output)
@@ -975,6 +1011,10 @@ class ContinuousBatchingSpyreModelRunner(SpyreModelRunner):
         prefill_index = self.input_batch.add_request(req_state)
         self.prefill_batch.add_request(req_state)
 
+        # set prefill index for logits processor
+        for logitsproc in self.input_batch.logitsprocs_wrappers:
+            logitsproc.set_prefill_index(prefill_index)
+
         # Refresh sampling metadata after all request are added to the batch
         self.input_batch.refresh_metadata()
         self.prefill_batch.refresh_metadata()
@@ -1223,8 +1263,13 @@ class ContinuousBatchingSpyreModelRunner(SpyreModelRunner):
             is_prefill=model_input.is_prompt)
 
     def get_sampling_metadata(self, is_prefill: bool) -> SamplingMetadata:
-        return self.prefill_batch.sampling_metadata \
-            if is_prefill else self.input_batch.sampling_metadata
+
+        if is_prefill:
+            sampling_data = self.prefill_batch.sampling_metadata
+            sampling_data.logitsprocs = self.input_batch.logitsprocs
+            return sampling_data
+        else:
+            return self.input_batch.sampling_metadata
 
     def get_req_id_to_index(self, is_prefill: bool) -> dict[str, int]:
         req_id_to_index = self.prefill_batch.get_unpadded_output_indices() \
@@ -1306,6 +1351,29 @@ class ContinuousBatchingSpyreModelRunner(SpyreModelRunner):
             torch._dynamo.mark_static(model_input.input_positions,
                                       1)  # always 1
 
+    def build_input_batch(self) -> SamplingInputBatch:
+        # Define logits processors.
+
+        custom_logitsprocs = self.vllm_config.model_config.logits_processors
+
+        batch_size = self.scheduler_config.max_num_seqs
+        logits_processors = \
+            build_logitsprocs_for_cb(vllm_config=self.vllm_config,
+                            device=self.device,
+                            is_pin_memory=self.pin_memory,
+                            is_pooling_model=False,
+                            custom_logitsprocs=custom_logitsprocs,
+                            batch_size=batch_size)
+
+        return SamplingInputBatch(
+            max_num_reqs=batch_size,
+            max_model_len=self.model_config.max_model_len,
+            device=self.device,
+            pin_memory=self.pin_memory,
+            vocab_size=self.model_config.get_vocab_size(),
+            logitsprocs=logits_processors,
+        )
+
 
 class PoolerAdapter(torch.nn.Module):
 
@@ -1316,16 +1384,20 @@ class PoolerAdapter(torch.nn.Module):
     def forward(
         self,
         hidden_states: Union[torch.Tensor, list[torch.Tensor]],
-        *args,
+        pooling_metadata: PoolingMetadata,
     ) -> Union[torch.Tensor, list[torch.Tensor]]:
+        # Because we're using transformers to load the pooler
+        # and classifier layers and the assumption there is that
+        # we have a right padded batch, we need to split
+        # and at the batch dimension.
         if isinstance(hidden_states, torch.Tensor):
-            return self.pooler(hidden_states.T)
-        else:
-            return [self.pooler(h.T) for h in hidden_states]
+            hidden_states = torch.split(hidden_states,
+                                        pooling_metadata.prompt_lens.tolist())
+        return [self.pooler(h.unsqueeze(dim=0)) for h in hidden_states]
 
 
-def _transpose(input: torch.Tensor) -> torch.Tensor:
-    return input.T
+def _cls(input: torch.Tensor) -> torch.Tensor:
+    return input[:, 0]
 
 
 class SpyrePoolingModelRunner(WarmupShapesMixin,
@@ -1378,7 +1450,7 @@ class SpyrePoolingModelRunner(WarmupShapesMixin,
                 self._pooler = PoolerAdapter(self.model.pooler)
             elif hasattr(class_model, "roberta"):
                 self.model = class_model.roberta
-                self._pooler = PoolerAdapter(_transpose)
+                self._pooler = PoolerAdapter(_cls)
             else:
                 raise ValueError(
                     f"Unsupported model {self.model_config.model}: Expected "
@@ -1387,6 +1459,11 @@ class SpyrePoolingModelRunner(WarmupShapesMixin,
         else:
             raise ValueError(f"Unsupported task {task}")
 
+        # Disable pooler because in transformers it's
+        # always run even tough we don't use the outputs
+        # directly.
+        self.model.pooler = None
+
         model_class_name = type(self.model).__name__
         self.is_roberta = "roberta" in model_class_name.lower()
 
@@ -1419,29 +1496,17 @@ class SpyrePoolingModelRunner(WarmupShapesMixin,
 
         pooler_config = self.model_config.pooler_config
 
-        # TODO: remove this when we no longer support vllm version pre this
-        # PR https://github.com/vllm-project/vllm/pull/20538 (post v0.10.0)
-        annotations = inspect.getfullargspec(Pooler.for_embed).annotations
-        extra_args = {}
-        if ('default_normalize' in annotations
-                and 'default_softmax' in annotations):
-            extra_args.update({
-                'default_normalize': True,
-                'default_softmax': False
-            })
-        if 'default_pooling_type' in annotations:
-            extra_args['default_pooling_type'] = PoolingType.CLS
-
         if task == "embed":
-            self.pooler = Pooler.for_embed(pooler_config=pooler_config,
-                                           **extra_args)
+            with set_current_vllm_config(self.vllm_config):
+                self.pooler = Pooler.for_embed(pooler_config=pooler_config)
         elif task == "classify":
-            self.pooler = ClassifierPooler(
-                pooling=self._pooler,
-                classifier=self.classifier,
-                act_fn=ClassifierPooler.act_fn_for_cross_encoder(
-                    self.model_config),
-            )
+            with set_current_vllm_config(self.vllm_config):
+                self.pooler = ClassifierPooler(
+                    pooling=self._pooler,
+                    classifier=self.classifier,
+                    act_fn=ClassifierPooler.act_fn_for_cross_encoder(
+                        self.model_config),
+                )
 
     @property
     def vocab_size(self) -> int:
@@ -1470,10 +1535,37 @@ class SpyrePoolingModelRunner(WarmupShapesMixin,
                 self.input_batch.remove_request(req_id)
                 self.requests.pop(req_id, None)
 
+    def _uncompress_token_types(self) -> list[list[int]]:
+
+        pooling_metadata = self.input_batch.make_pooling_metadata()
+        pooling_params = pooling_metadata.pooling_params
+
+        token_type_id_requests = dict[int, Any]()
+        for i, param in enumerate(pooling_params):
+            if param.extra_kwargs is not None and \
+            (token_types := param.extra_kwargs.get(
+                "compressed_token_type_ids")) is not None:
+                token_type_id_requests[i] = token_types
+
+        if len(token_type_id_requests) == 0:
+            return []
+
+        seq_lens = pooling_metadata.prompt_lens
+        token_type_ids = []
+
+        for i, seq_len in enumerate(seq_lens):
+            pos = token_type_id_requests.get(i, seq_len)
+            ids = (torch.arange(seq_lens[i]) >= pos).int()
+            token_type_ids.append(ids)
+
+        return token_type_ids
+
     def _token_types(self, input_ids):
-        from vllm.model_executor.models import bert
-        if hasattr(bert, "_decode_token_type_ids"):
-            return bert._decode_token_type_ids(input_ids)
+        if (token_type_ids_lst := self._uncompress_token_types()):
+            token_type_ids = torch.zeros_like(input_ids)
+            for i, token_types in enumerate(token_type_ids_lst):
+                token_type_ids[i, -len(token_types):] = token_types
+            return token_type_ids
         else:
             locs = torch.where(input_ids == self.sep_token_id, 1, 0)
             return locs.cumsum(dim=1) - locs
@@ -1616,6 +1708,13 @@ class SpyrePoolingModelRunner(WarmupShapesMixin,
 
         pooling_metadata = self.input_batch.make_pooling_metadata()
 
+        # temporary backward compat code for 0.10.1.1
+        if getattr(pooling_metadata, "build_pooling_cursor", False):
+            ## No partial prefill, hence we can use the prompt lens here
+            pooling_metadata.build_pooling_cursor(
+                num_scheduled_tokens=pooling_metadata.prompt_lens,
+                device=self.device)
+
         # prepare unpadded output for the pooler
         hidden_state_list: list[torch.Tensor] = []
         for hidden_state, prompt_len in zip(hidden_states,
@@ -1623,21 +1722,29 @@ class SpyrePoolingModelRunner(WarmupShapesMixin,
             # we're left padding
             hidden_state_list.append(hidden_state[-prompt_len:])
 
-        raw_pooler_output = self.pooler(hidden_states=hidden_state_list,
-                                        pooling_metadata=pooling_metadata)
+        raw_pooler_output = self.pooler(
+            hidden_states=torch.cat(hidden_state_list),
+            pooling_metadata=pooling_metadata)
 
         pooler_output: list[Optional[torch.Tensor]] = []
 
         for raw_output in raw_pooler_output:
             pooler_output.append(raw_output.data.to("cpu"))
 
+        # temporary backward compat code for 0.10.1.1
+        annotations = inspect.getfullargspec(ModelRunnerOutput).annotations
+        extra_args = {}  # type: ignore
+        if ('spec_token_ids' in annotations):
+            extra_args.update({
+                'spec_token_ids': None,
+            })
+
         model_output = ModelRunnerOutput(
             req_ids=self.input_batch.requests_ids,
             req_id_to_index=self.input_batch.req_id_to_index,
             sampled_token_ids=[],
-            spec_token_ids=None,
             logprobs=None,
             prompt_logprobs_dict={},
             pooler_output=pooler_output,
-        )
+            **extra_args)
         return model_output
